package especialista;

import java.util.*;
import javax.ejb.*;
import javax.naming.InitialContext;
import java.rmi.RemoteException;
import javax.rmi.PortableRemoteObject;

public class CartBean implements SessionBean {
   static BookFinderHome home = null;
   static InitialContext ic = null;

   String customerName;
   String customerId;
   Collection contents;
   double precio;
   BookFinder bookFinder;

   public void ejbCreate(String person) throws CreateException {
      initializeSingletons();
      if (person == null) {
         throw new CreateException("Null person not allowed.");
      } else {
         customerName = person;
      }
      customerId = "0";
      contents = new ArrayList();
      try {
         bookFinder = home.create();
      } catch (RemoteException ex) {
         throw new CreateException("Error al crear el bean bookFinder");
      }
   }

   public void ejbCreate(String person, String id)
      throws CreateException {
      initializeSingletons();
      if (person == null) {
         throw new CreateException("Null person not allowed.");
      } else {
         customerName = person;
      }
      IdVerifier idChecker = new IdVerifier();
      if (idChecker.validate(id)) {
         customerId = id;
      } else {
         throw new CreateException("Invalid id: " + id);
      }
      contents = new ArrayList();
      try {
         bookFinder = home.create();
      } catch (RemoteException ex) {
         throw new CreateException("Error al crear el bean bookFinder");
      }
   }

   private void initializeSingletons() throws CreateException {
      if (ic == null) {
         try {
            ic = new InitialContext();
            Object objref = ic.lookup("BookFinderBean");
            home =
               (BookFinderHome) PortableRemoteObject.narrow(
                  objref,
                  BookFinderHome.class);
         } catch (Exception ex) {
            throw new CreateException("Excepcion al buscar el bean BookFinderBean");
         }
      }
   }

   public void addBook(String title) throws BookException {
     BookVO book;
     
     try {
		book = bookFinder.getBook(title);
     } catch (RemoteException ex) {
     	  throw new BookException("Problemas con la red: " +
     	                          ex.getMessage());
     }
      contents.add(book);
   }

   public void removeBook(String title) throws BookException {
      boolean result = false;

      Iterator it = contents.iterator();
      while (it.hasNext()) {
         BookVO book = (BookVO) it.next();
         if (title.equals(book.getTitle())) {
            it.remove();
            result = true;
            break;
         }
      }
      if (result == false) {
         throw new BookException(title + " not in cart.");
      }
   }

   public ArrayList getBookTitles() {
      ArrayList returnList = new ArrayList();

      Iterator it = contents.iterator();
      while (it.hasNext()) {
         BookVO book = (BookVO) it.next();
         returnList.add(book.getTitle());
      }
      return returnList;
   }

	public ArrayList getBookIsbns() {
		ArrayList returnList = new ArrayList();

		Iterator it = contents.iterator();
		while (it.hasNext()) {
			BookVO book = (BookVO) it.next();
			returnList.add(book.getIsbn());
		}
		return returnList;
	}
	
   public double getOrderPrice() throws RemoteException {
      double price = 0.0;
      double discountLevel, discountPercent;
      Double dbl1, dbl2;

      try {
         ic = new InitialContext();
         dbl1 = (Double) ic.lookup("java:comp/env/DiscountLevel");
         dbl2 = (Double) ic.lookup("java:comp/env/DiscountPercent");
      } catch (Exception ex) {
         throw new RemoteException("Error al buscar las entradas del entorno");
      }

      discountLevel = dbl1.doubleValue();
      discountPercent = dbl2.doubleValue();

      Iterator it = contents.iterator();
      while (it.hasNext()) {
         BookVO book = (BookVO) it.next();
         price = price + book.getPrice();
      }

      if (price >= discountLevel)
         price = price * (1 - discountPercent);
      return price;
   }

   public CartBean() {
   }
   public void ejbRemove() {
   }
   public void ejbActivate() {
   }
   public void ejbPassivate() {
   }
   public void setSessionContext(SessionContext sc) {
   }
}
